<?php 

namespace PrestaShop\Module\LCVPrestaConnector\Controller;

use PrestaShop\Module\LCVPrestaConnector\Forms\GrilleType;
use PrestaShop\Module\LCVPrestaConnector\Forms\InformationType;
use PrestaShop\Module\LCVPrestaConnector\Forms\CatalogType;
use PrestaShop\Module\LCVPrestaConnector\Forms\OrderType;
use Symfony\Component\HttpFoundation\Request;
use PrestaShop\Module\LCVPrestaConnector\SyncConfiguration;
use PrestaShop\Module\LCVPrestaConnector\Syncer;

class ConfigurationAdminController extends AdminController
{
    /**
     * Indique si la resynchronisation est en cours
     */
    private ?bool $isResyncing = null;

    /**
     * Obtient la description de la barre d'outils de l'interface d'administration
     */
    private function getAdminLayoutToolBar() : array 
    {
        $toolbar = [
                'bind' => [
                    'href' => $this->generateUrl($this->module->name . '_configuration') . '&reset-binding=1',
                    'class' => 'btn-outline-secondary',
                    'desc' => 'Déconnecter',
                    'help' => '⚠⚠⚠ Déconnecte le module de votre backoffice. Vous devrez vous reconnecter pour continuer à utiliser le module.',
                    'icon' => 'cancel',                    
                ],
                'resync' => [
                    'href' => '#',
                    'class' => 'btn-outline-primary',
                    'desc' => 'Resynchroniser',
                    'help' => 'Programme une synchronisation totale la prochaine fois que la tâche de synchronisation est lancée.',
                    'icon' => 'refresh',                    
                    'js' => $this->module->name . '_resyncAll();',
                ],
                'run' => [
                    'href' => '#',
                    'class' => 'btn-outline-primary',
                    'desc' => '',
                    'help' => 'Déclenche la tâche des synchronisation des données.',
                    'icon' => 'play_circle',
                    'js' => $this->module->name . '_run();',
                ],
            ];
                
        if ($this->isResyncing === null)
        {
            // Extraction de la position de synchronisation
            $positions = $this->module->getCfg()->get(SyncConfiguration::CFG_SYNC_POSITIONS);
            $this->isResyncing = empty($positions);
        }

        if ($this->isResyncing)
        {
            $toolbar['resync']['desc'] = 'Resynchro. demandée !';
            $toolbar['resync']['class'] = 'btn-outline-secondary disabled';
        }

        return $toolbar;
    }

    /**
     * Retourne les variables passées en paramètre enrichies des variables par défaut
     */
    private function addDefaultVars(array $vars) : array
    {
        if (!isset($vars['layoutHeaderToolbarBtn']))
            $vars['layoutHeaderToolbarBtn'] = $this->getAdminLayoutToolBar();

        if (!isset($vars['backoffice']))
            $vars['backoffice'] = $this->module->getBridge()->getId();
        
        return $vars;
    }


    public function grilleDeTaille(Request $request){
     
        $form = $this->createForm(GrilleType::class);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            //$data = $form->getData();
        }
    
        return $form;

    }

    public function informationForm(Request $request){
        
        $form = $this->createForm(InformationType::class);
        
        $form->handleRequest($request);

        if($form->isSubmitted() && $form->isValid()){
            $cfg = $this->module->getCfg();

            // on enregistre directement dans la bdd si mode est vrai alors la valeur et à 0 sinon elle est à 1 
            $cfg->set(SyncConfiguration::CFG_ACTIVE,  $form->get("synchro_active")->getData() ? true : false ); 

            // on enregistre directement dans la bdd si mode est vrai alors la valeur et à 0 sinon elle est à 1 
            $cfg->set(SyncConfiguration::CFG_WORKMODE, $form->get("cfg_mode")->getData() ? 0 : 1 ); 

            // token guard ?
            $cfg->set(SyncConfiguration::CFG_GUARD_TOKEN, $form->get("guard_token")->getData() );

            $this->addFlash('success', 'Configuration sauvegardée !');
        }

        return $form;

    }

    public function catalogForm(Request $request){

        $cfg =  $this->module->getCfg();
        $form = $this->createForm(CatalogType::class);

        $form->handleRequest($request);

        if($form->isSubmitted() && $form->isValid()){

            $data = $form->getData();

            // Option du stock sécurisé
            // On vérifie que le stock sécurisé est un entier, autrement erreur                        
            if (isset($data['secured_stock']))
            {
                if (!is_numeric($data["secured_stock"]) || $data["secured_stock"] < 0)
                {
                    $this->addFlash('error', "Le stock sécurisé doit être un nombre entier positif.");
                    return $form;
                }
                $cfg->set(SyncConfiguration::CFG_SECURED_STOCK, $data["secured_stock"]);
            }

            // L'option de visibilité des produits ...
            if (isset($data['hide_stock_method']))
                $cfg->set(SyncConfiguration::CFG_NO_STOCK_PRODUCT_VISIBILITY, $data["hide_stock_method"]);

            // Récupérer uniquement les champs correspondant à $cfgOptions
            $cfgOptions = $cfg->get(SyncConfiguration::CFG_DATA_SYNC);

            // Filtrer les données pour ne garder que les champs autorisés
            foreach (array_keys($this->module->getCfg()->describeSyncConfiguration()) as $key) {
                if (isset($data[$key])) {
                    $value = $data[$key];
                    if (!is_bool($value))
                    {
                        $this->addFlash('error', "La valeur de '$key' doit être vrai ou faux.");
                        return $form;
                    }
                    $cfgOptions[$key] = $value;
                }                
            }

            // Sauvegarder uniquement les données filtrées dans la configuration
            $cfg->set(SyncConfiguration::CFG_DATA_SYNC, $cfgOptions);

            /**
             * Mapping des attributs de produits
             */
            $productMapping = $cfg->get(SyncConfiguration::CFG_PR_ATTRIBUTES_MAPPING);
            foreach (array_merge($cfg->get(SyncConfiguration::CFG_DYN_ATTR), $cfg->get(SyncConfiguration::CFG_DYN_CAT)) as $k => $name)
            {
                $base_safe_key = trim(base64_encode($k), '=');
                
                if (isset($data['PRODUCT_ATTR_PRODUCT_'.$base_safe_key]))
                    $productMapping[SyncConfiguration::PRODUCT_ATTR_PRODUCT][$k] = $data['PRODUCT_ATTR_PRODUCT_'.$base_safe_key];

                if (isset($data['PRODUCT_ATTR_VARIANT_'.$base_safe_key]))
                    $productMapping[SyncConfiguration::PRODUCT_ATTR_VARIANT][$k] = $data['PRODUCT_ATTR_VARIANT_'.$base_safe_key];

                if (isset($data['PRODUCT_ATTR_FEATURE_'.$base_safe_key]))
                    $productMapping[SyncConfiguration::PRODUCT_ATTR_FEATURE][$k] = $data['PRODUCT_ATTR_FEATURE_'.$base_safe_key];
            }
            $cfg->set(SyncConfiguration::CFG_PR_ATTRIBUTES_MAPPING, $productMapping);


            $this->addFlash('success', 'Configuration sauvegardée !');
        }

        return $form;
    }

    /**
     * Gestion du formulaire de configuration des commandes
     */
    public function orderForm(Request $request){

        $cfg =  $this->module->getCfg();
        $form = $this->createForm(OrderType::class);

        $form->handleRequest($request);

        if($form->isSubmitted() && $form->isValid()){

            $data = $form->getData();

            /** Export/Import client */
            if (isset($data['customer_import']))
                $cfg->set(SyncConfiguration::CFG_IMPORT_CUSTOMERS, (bool) ($data["customer_import"] ?? false));
            if (isset($data['customer_export']))
                $cfg->set(SyncConfiguration::CFG_EXPORT_CUSTOMERS, (bool) ($data["customer_export"] ?? false));

            // Référence à utiliser pour les frais de port et frais d'emballage
            if (isset($data['shipping_ref']))
                $cfg->set(SyncConfiguration::CFG_SHIPPING_COST_REF, $data["shipping_ref"] ?? 'shipping');

            if (isset($data['wrapping_ref']))
                $cfg->set(SyncConfiguration::CFG_WRAPPING_COST_REF, $data["wrapping_ref"] ?? 'wrapping');

            // Les modes de règlements ...
            // On va préablement récupérer le mapping existant
            $mapReg = $cfg->get(SyncConfiguration::CFG_ORDER_PAY_MAPPAGE);

            // Puis, nous allons créer une option pour chaque mode de règlement, de type text
            foreach (array_keys($mapReg) as $key) {
                // On encode la clé avec base64 sans = pour éviter les problèmes de clé
                $safe_key = 'map_reg_'. rtrim(base64_encode($key), '=');
                // On récupère la valeur dans le formulaire
                if (isset($data[$safe_key]))
                {
                    // On met à jour la valeur
                    $mapReg[$key] = $data[$safe_key];                    
                }
            }

            // On enregistre
            $cfg->set(SyncConfiguration::CFG_ORDER_PAY_MAPPAGE, $mapReg);

            // Les bons ...
            $optVouchers = $cfg->get(SyncConfiguration::CFG_VOUCHER_IMPORT_TYPES);
            // Filtrer les données pour ne garder que les champs autorisés
            foreach (array_keys($this->module->getCfg()->describeVoucherConfiguration()) as $key) {
                if (isset($data['import_voucher_'.$key])) {
                    $value = $data['import_voucher_'.$key];
                    if (!is_bool($value))
                    {
                        $this->addFlash('error', "La valeur de 'import_voucher_$key' doit être vrai ou faux.");
                        return $form;
                    }
                    $optVouchers[$key] = $value;
                }                
            }
            $cfg->set(SyncConfiguration::CFG_VOUCHER_IMPORT_TYPES, $optVouchers);


            $this->addFlash('success', 'Configuration sauvegardée !');
        }

        return $form;
    }


    /**
     * Vérifie que le backoffice est bien jumelé ou renvoi null
     */
    public function checkLogin() : \Symfony\Component\HttpFoundation\Response|null
    {
        $loginCfg = $this->module->getCfg()->get(SyncConfiguration::CFG_LOGIN);

        if (empty($loginCfg) || (isset($loginCfg["error"]) && $loginCfg["error"]))
        {
            $specifics = $this->module->getBridge()->getSpecifics();            
            
            return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/login.html.twig', [
                'MODULE_NAME' => $this->module->name,
                'BACKOFFICE_NAME' => $this->module->getBridge()->getId(),
                'ADDRESS_LABEL' => $specifics["ADDRESS_LABEL"] ?? "",
                'PREVIOUS_ADDRESS' => $specifics["PREVIOUS_ADDRESS"] ?? "",
                "INITIAL_ERROR" => (isset($loginCfg["error"]) && $loginCfg["error"]) ? $loginCfg["error"] : "Il semble que le module n'est pas encore associé avec votre backoffice.",
            ]);            
        }

        return null;
    }

    /**
     * Rendu de la page de configuration des options générales
     */
    public function informationAction(Request $request) {
       
        // On vérifie s'il faut remonter les hooks
        if (isset($_REQUEST["remount_hook"]))
			$this->module->registerHooks();
           
        // On vérifie s'il faut réinitialiser le jumelage
        if (isset($_REQUEST["reset-binding"]) && $_REQUEST["reset-binding"])
        {
            // Oui, on réinitialise le jumelage en supprimant la configuration de login
            $cfgLogin = null;
            $this->module->getCfg()->set(SyncConfiguration::CFG_LOGIN, $cfgLogin);
        }
       

        // Vérifions que le jumelage est déjà fait !
        $bindPage = $this->checkLogin(); if ($bindPage) return $bindPage;       

        // On vérifie s'il faut programmer une resynchronisation totale
        if (isset($_REQUEST["reset-resync"]) && $_REQUEST["reset-resync"])
        {
            // On instancie un syncer
            $syncer = new Syncer($this->module);

            // Demande de resynchronisation totale pour la prochaine fois
            $syncer->bridge->resetSyncPosition();
        }
      

        $form = $this->informationForm($request);
              
        return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/information.html.twig', $this->addDefaultVars([
            'form' => $form->createView()
        ]));
    }

    /**
     * Rendu de la page de configuration des exportations de commandes
     */
    public function commandeAction(Request $request) {
        // Vérifions que le jumelage est déjà fait !
        $bindPage = $this->checkLogin(); if ($bindPage) return $bindPage;
  
        $form = $this->orderForm($request);

        return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/commande.html.twig', $this->addDefaultVars([
            'form' => $form->createView()
        ]));
    }


    /**
     * Rendu de la page de configuration du catalogue
     */
    public function catalogueAction(Request $request) {
        // Vérifions que le jumelage est déjà fait !
        $bindPage = $this->checkLogin(); if ($bindPage) return $bindPage;

        $form = $this->catalogForm($request);
    
        return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/catalogue.html.twig', $this->addDefaultVars([
            'form' => $form->createView()
        ]));
    }


   

}